@@ -558,7 +558,7 @@ class LinuxDevice(object):
 
         count = int(timeout * 10)
         ratio = float(total) / count
-        print("%s %d seconds" %(description, timeout))
+        self.log("%s %d seconds" %(description, timeout))
         # You can use this flag to run as a thread
         self.terminate_loop = False
         for i in range(1, count + 1):
