@@ -37,39 +37,34 @@ class OvsHost(linux.LinuxDevice):
         self.macs = []
 
         # start the docker if it's not running
-        num = self.cmd('ps -ef | grep -w ovs-vswitchd | grep -v grep | wc -l')
-        if num and int(num) <= 0:
-            self.cmd('ovs-ctl start')
-            self.cmd('ovs-vsctl --no-wait set Open_vSwitch . other_config:dpdk-init=true')
-            self.log("Setup OVS complete")
+        self.cmd('ovs-ctl start')
+        self.cmd('systemctl start docker')
 
-        self["vlog"] = self["vlog"] and int(self["vlog"]) or -1
-        if (self["vlog"] >= 0):
+        #vlog: 0: no change. 1: emer, 2: err, 3: warn, 4: info, 5: dbg, -1: off
+        self["vlog"] = (self["vlog"]) and int(self["vlog"]) or 0
+        if (self["vlog"] > 0):
             self.cmd('echo > /usr/local/var/log/openvswitch/ovs-vswitchd.log')
-            #self.cmd('echo > /usr/local/var/log/openvswitch/ovsdb-server.log')
-            if (self["vlog"] == 0):
-                self.cmd('ovs-appctl vlog/set ANY:file:emer')
-            elif (self["vlog"] == 1):
-                self.cmd('ovs-appctl vlog/set ANY:file:err')
-            elif (self["vlog"] == 2):
-                self.cmd('ovs-appctl vlog/set ANY:file:warn')
-            elif (self["vlog"] == 3):
-                self.cmd('ovs-appctl vlog/set ANY:file:info')
-            elif (self["vlog"] >= 4):
-                self.cmd('ovs-appctl vlog/set ANY:file:dbg')
-
-        # start the docker if it's not running
-        num = self.cmd('ps -ef | grep -w dockerd | grep -v grep | wc -l')
-        if num and int(num) <= 0:
-            self.cmd('systemctl start docker')
-        self.log("Setup docker complete")
+        #self.cmd('echo > /usr/local/var/log/openvswitch/ovsdb-server.log')
+        if (self["vlog"] == -1):
+            self.cmd('ovs-appctl vlog/set ANY:ANY:off')
+        elif (self["vlog"] == 1):
+            self.cmd('ovs-appctl vlog/set ANY:file:emer')
+        elif (self["vlog"] == 2):
+            self.cmd('ovs-appctl vlog/set ANY:file:err')
+        elif (self["vlog"] == 3):
+            self.cmd('ovs-appctl vlog/set ANY:file:warn')
+        elif (self["vlog"] == 4):
+            self.cmd('ovs-appctl vlog/set ANY:file:info')
+        elif (self["vlog"] >= 5):
+            self.cmd('ovs-appctl vlog/set ANY:file:dbg')
+        #self.cmd('ovs-appctl vlog/list')
 
     def __del__(self):
         '''
         Get the trace file.
         Don't stop the OVS or docker.
         '''
-        if (self["vlog"] >= 0):
+        if (self["vlog"] > 0):
             #self.get_file('/usr/local/var/log/openvswitch/ovsdb-server.log',
             #        "%s_ovsdb-server.log" %(self["name"]))
             self.get_file('/usr/local/var/log/openvswitch/ovs-vswitchd.log',
@@ -199,7 +194,6 @@ class OvsHost(linux.LinuxDevice):
             else:
                 self.log("device type %s is not supported now..." %(dtype))
 
-
     def __add_container (self, bridge_name,
             name = "con1",
             host_dir = "/var/shared",
@@ -262,62 +256,56 @@ class OvsHost(linux.LinuxDevice):
         for k,v in vm.items():
             if k[0] == "-":
                 cmd += ' %s %s' %(k,v)
-
         for p in vm.get("port", []):
             for k,v in p.items():
                 if k[0] == "-":
                     #bypass the parameters: name, ip
                     cmd += ' %s %s' %(k,v)
 
+        # 1) Add vhostuser port
+        for p in vm.get("port", []):
+            device = self._util_dictionary_parse(p["-device"].split(","))
             if "vhost-user" in p["-netdev"]:
                 # type: vhostuserclient
-                # Parse the parameter for -chardev/-netdev/-device
-                chardev = {}
-                for attribute in p["-chardev"].split(","):
-                    if ("=" in attribute):
-                        k,v = attribute.split("=", 1)
-                        chardev[k.strip()] = v.strip()
-                    else:
-                        chardev[attribute] = ""
-
-                netdev = {}
-                for attribute in p["-netdev"].split(","):
-                    if ("=" in attribute):
-                        k,v = attribute.split("=", 1)
-                        netdev[k.strip()] = v.strip()
-                    else:
-                        netdev[attribute] = ""
-
-                device = {}
-                for attribute in p["-device"].split(","):
-                    if ("=" in attribute):
-                        k,v = attribute.split("=", 1)
-                        device[k.strip()] = v.strip()
-                    else:
-                        device[attribute] = ""
-
-                #print("chardev: %s" %(chardev))
-                #print("netdev: %s" %(netdev))
-                #print("device: %s" %(device))
-                p["mac"] = device["mac"] # Record the mac for use later
-                port_name = "to_%s_%s" %(vm.get("name", "tmp"), netdev["id"])
+                chardev = self._util_dictionary_parse(p["-chardev"].split(","))
+                port_name = "to_%s_%s" %(vm["-name"], os.path.basename(chardev["path"]))
                 ovs_port = {"name": port_name,
                         "type": "dpdkvhostuserclient",
                         "options:vhost-server-path": chardev["path"]}
                 self.add_port(bridge_name, ovs_port)
-                self.macs.append(p["mac"])
-                self.devices.append(p)
-                # Configure offlow for the same subnet
-                if self["fail-mode"] == "secure":
-                    p["ofport"] = self.ovs_get_interface(port_name, "ofport")
-                    ipaddr = ipaddress.ip_interface(p["ip"])
-                    self.ofctl_add_flow(bridge_name,
-                        "priority=65535,arp,arp_tpa=%s,actions=output:%s"
-                        %(ipaddr.ip, p["ofport"]),
-                        "priority=60000,ip,nw_src=%s,nw_dst=%s,actions=output:%s" 
-                        %(ipaddr.network, ipaddr.ip, p["ofport"]))
+
+        # 2)start the qemu: tap mode need to start the qemu at frist,
+        # vhostuser need to start the ovs-port at first. 
         tmp = self.cmd("%s -daemonize" %(cmd))
-        #val = os.system(cmd + "&")
+
+        # 3)add the tap port, and add the flow
+        for p in vm.get("port", []):
+            device = self._util_dictionary_parse(p["-device"].split(","))
+            if "vhost-user" in p["-netdev"]:
+                # type: vhostuserclient, add the port before qemu command.
+                print("Have added.")
+            elif "tap" in p["-netdev"]:
+                # type: tap
+                netdev = self._util_dictionary_parse(p["-netdev"].split(","))
+                port_name = netdev.get("ifname", "tmp")
+                ovs_port = {"name": port_name}
+                self.add_port(bridge_name, ovs_port)
+            else:
+                # Now only vhost-user and tap will be added into OVS
+                continue
+
+            p["mac"] = device["mac"] # Record the mac for use later
+            self.macs.append(p["mac"])
+            self.devices.append(p)
+            # Configure offlow for the same subnet
+            if self["fail-mode"] == "secure":
+                p["ofport"] = self.ovs_get_interface(port_name, "ofport")
+                ipaddr = ipaddress.ip_interface(p["ip"])
+                self.ofctl_add_flow(bridge_name,
+                    "priority=65535,arp,arp_tpa=%s,actions=output:%s"
+                    %(ipaddr.ip, p["ofport"]),
+                    "priority=60000,ip,nw_src=%s,nw_dst=%s,actions=output:%s" 
+                    %(ipaddr.network, ipaddr.ip, p["ofport"]))
 
     def add_vtep (self, bridge_name, vtep, *args):
         '''
@@ -507,6 +495,21 @@ class OvsHost(linux.LinuxDevice):
                 self.log("FAIL: %s ping %s, %s -> %s!" %(src["name"], dst["name"],
                     srcip.ip, dstip.ip), bg_color = "red")
 
+    def _util_dictionary_parse (self, params):
+        '''
+        Add some flows to ofproto
+        '''
+        my_dic = {}
+        if "=" not in params[0]:
+            my_dic["type"] = params.pop(0)
+        for attribute in params:
+            if ("=" in attribute):
+                k,v = attribute.split("=", 1)
+                my_dic[k.strip()] = v.strip()
+            else:
+                my_dic[attribute] = ""
+        return my_dic
+
 if __name__ == '__main__':
     '''
     #topology：
